
(defun zorch-line ()
  "Kill the line the point is on, from anywhere on that line"
  (interactive)
  (save-excursion
    (let ((start (line-beginning-position))
	  (end (if (= (line-end-position) (point-max))
		   (line-end-position)
		 (+ 1 (line-end-position)))))
      (kill-new (buffer-substring start end))
      (delete-region start end))
    ))

(defun duplicate-line ()
  "Duplicate the line the point is on, moving the point to the duplicate"
  (interactive)
  (save-excursion
    (kill-new (buffer-substring (line-beginning-position) (line-end-position)))
    (end-of-line)
    (newline)
    (yank)
    (rotate-yank-pointer 1)
    )
  (next-line 1)
  )

(defun entag (start end tagname)
  "Surround region with HTML tag"
  (interactive "r\nsTag name: ")
  (save-excursion
    (goto-char end)
    (insert "</" tagname ">")
    (goto-char start)
    (insert "<" tagname ">")
    )
  (goto-char (+ start 1 (length tagname)))
)

(defun insert-tag (tagname)
  "Insert tag around point"
  (interactive "sTag name: ")
  (insert "<" tagname
	  (cond
	   ((equal tagname "a") " href=\"\"")
	   ((equal tagname "div") " id=\"\"")
	   ((equal tagname "img") " src=\"\"")
	   ((equal tagname "script") " type=\"text/javascript\"")
	   (t ""))
	  ">")
  (let ((final-location (point)))
    (insert "</" tagname ">")
    (goto-char final-location)))

(defun copy-whole-buffer ()
  "Add a new entry to the kill ring containing the entire buffer"
  (interactive)
  (kill-new (buffer-substring (point-min) (point-max)))
  (message "Buffer %s copied to kill ring" (buffer-name)))

(defun roll-line-down ()
  "Move current line downward once."
  (interactive)
  (let (col (current-column))
    (forward-line)
    (transpose-lines 1)
    (forward-line -1)
    (forward-char col)))

(defun roll-line-up ()
  "Move current line upward once."
  (interactive)
  (let ((col (current-column)))
    (transpose-lines 1)
    (forward-line -2)
    (forward-char col)))

(defun banner (text)
  "Insert a banner comment at point"
  (interactive "sBanner text: ")
  (let ((col (current-column)))
    (insert "##################################################\n")
    (while (< (current-column) col)
      (insert " "))
    (insert "### " text " ")
    (while (< (current-column) (+ 50 col))
      (insert "#"))
    (insert "\n")
    (while (< (current-column) col)
      (insert " "))
    (insert "##################################################\n")))

(defun destroy-frame ()
  (interactive)
  (kill-buffer (current-buffer))
  (delete-window))

(defun kill-all-buffers ()
  (interactive)
  (mapcar (lambda (b) (kill-buffer b)) (buffer-list))
  (delete-other-windows))

(defun close-script-tag-idx ()
  (save-excursion
    (search-forward "</script>")
    (backward-char 9) ; length of </script>
    (point)))

(defun open-script-tag-idx ()
  (save-excursion
    (search-backward "<script")
    (search-forward ">")
    (point)))

(defun narrow-to-script ()
  (interactive)
  (narrow-to-region (open-script-tag-idx)
		    (close-script-tag-idx))
  (ecmascript-mode))

(global-set-key [(meta up)] 'roll-line-up)
(global-set-key [(meta down)] 'roll-line-down)
(global-set-key "\M-k" 'zorch-line)
(global-set-key "\M-d" 'duplicate-line)
(global-set-key "\C-e" 'insert-tag) 
(global-set-key "\C-a" 'copy-whole-buffer)
(global-set-key "\M- " 'dabbrev-expand)
(global-set-key "\C-xd" 'destroy-frame)
(global-set-key "\C-xK" 'kill-all-buffers)
(global-set-key "\C-xns" 'narrow-to-script)
